//------------------------------------------------------------------------------
// Lamp : Open source game middleware
// Copyright (C) 2004  Junpei Ohtani ( Email : junpee@users.sourceforge.jp )
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

/** @file
 * 二次元テクスチャ座標ヘッダ
 * @author Junpee
 */

#ifndef TEX_COORD_2_H_
#define TEX_COORD_2_H_

#include <Core/System/Math.h>
#include <Core/Primitive/String.h>

namespace Lamp{

//------------------------------------------------------------------------------
/**
 * 二次元テクスチャ座標
 *
 * このクラスは継承しないで下さい。
 */
class TexCoord2{
public:
	//--------------------------------------------------------------------------
	// メンバ変数
	//--------------------------------------------------------------------------
	/// メンバ変数
	union{
		/// 各要素
		struct{
			/// U値
			float u;
			/// V値
			float v;
		};

		/// 配列
		float array[2];
	};

	//--------------------------------------------------------------------------
	// 定数
	//--------------------------------------------------------------------------
	/// ゼロ座標
	static const TexCoord2 zero;

	/// 単位座標
	static const TexCoord2 unit;

	/// U単位座標
	static const TexCoord2 unitU;

	/// V単位座標
	static const TexCoord2 unitV;

	//--------------------------------------------------------------------------
	// コンストラクタ
	//--------------------------------------------------------------------------
	/**
	 * コンストラクタ
	 *
	 * このコンストラクタは初期値の設定を行わないため値は不定です。
	 */
	inline TexCoord2(){}

	/**
	 * コンストラクタ
	 * @param sourceU Uの初期値
	 * @param sourceV Vの初期値
	 */
	inline TexCoord2(float sourceU, float sourceV) :
		u(sourceU), v(sourceV){
	}

	/**
	 * コンストラクタ
	 * @param source 初期値配列
	 */
	inline explicit TexCoord2(const float* const source) :
		u(source[0]), v(source[1]){
	}

	//--------------------------------------------------------------------------
	// 値の設定
	//--------------------------------------------------------------------------
	/**
	 * 値の設定
	 * @param sourceU Uの設定値
	 * @param sourceV Vの設定値
	 */
	inline void set(float sourceU, float sourceV){
		u = sourceU;
		v = sourceV;
	}

	/**
	 * 値の設定
	 * @param source 設定値配列
	 */
	inline void set(const float* const source){
		u = source[0];
		v = source[1];
	}

	//--------------------------------------------------------------------------
	// 演算
	//--------------------------------------------------------------------------
	/**
	 * 加算
	 * @param addCoord 加算する二次元テクスチャ座標
	 * @return 加算された二次元テクスチャ座標
	 */
	inline TexCoord2 operator +(const TexCoord2& addCoord) const{
		return TexCoord2(u + addCoord.u, v + addCoord.v);
	}

	/**
	 * 減算
	 * @param subCoord 減算する二次元テクスチャ座標
	 * @return 減算された二次元テクスチャ座標
	 */
	inline TexCoord2 operator -(const TexCoord2& subCoord) const{
		return TexCoord2(u - subCoord.u, v - subCoord.v);
	}

	/**
	 * 乗算
	 * @param mulValue 乗算する値
	 * @return 乗算された二次元テクスチャ座標
	 */
	inline TexCoord2 operator *(float mulValue) const{
		return TexCoord2(u * mulValue, v * mulValue);
	}

	/**
	 * 乗算
	 * @param mulValue 乗算する値
	 * @param mulCoord 乗算する二次元テクスチャ座標
	 * @return 乗算された二次元テクスチャ座標
	 */
	inline friend TexCoord2 operator *(
		float mulValue, const TexCoord2& mulCoord){
		return TexCoord2(mulCoord.u * mulValue, mulCoord.v * mulValue);
	}

	/**
	 * +演算子
	 * @return 二次元テクスチャ座標のコピー
	 */
	inline TexCoord2 operator +() const{ return *this; }

	/**
	 * -演算子
	 * @return 値の符号が反転した二次元テクスチャ座標
	 */
	inline TexCoord2 operator -() const{ return TexCoord2(-u, -v); }

	//--------------------------------------------------------------------------
	// 代入演算
	//--------------------------------------------------------------------------
	/**
	 * 代入加算
	 * @param addCoord 加算する二次元テクスチャ座標
	 * @return 加算された二次元テクスチャ座標
	 */
	inline TexCoord2& operator +=(const TexCoord2& addCoord){
		u += addCoord.u;
		v += addCoord.v;
		return *this;
	}

	/**
	 * 代入減算
	 * @param subCoord 減算する二次元テクスチャ座標
	 * @return 減算された二次元テクスチャ座標
	 */
	inline TexCoord2& operator -=(const TexCoord2& subCoord){
		u -= subCoord.u;
		v -= subCoord.v;
		return *this;
	}

	/**
	 * 代入乗算
	 * @param mulValue 乗算する値
	 * @return 乗算された二次元テクスチャ座標
	 */
	inline TexCoord2& operator *=(float mulValue){
		u *= mulValue;
		v *= mulValue;
		return *this;
	}

	//--------------------------------------------------------------------------
	// 論理演算
	//--------------------------------------------------------------------------
	/**
	 * 二次元テクスチャ座標が同じかどうか
	 * @param target 比較する二次元テクスチャ座標
	 * @return 同じ値であればtrueを返す
	 */
	inline bool operator ==(const TexCoord2& target) const{
		return ((u == target.u) && (v == target.v));
	}

	/**
	 * 二次元テクスチャ座標が同じかどうか
	 * @param target 比較する二次元テクスチャ座標
	 * @param epsilon 誤差
	 * @return 誤差の範囲内で同じ値であればtrueを返す
	 */
	inline bool epsilonEquals(
		const TexCoord2& target, float epsilon) const{
		Assert(epsilon >= 0.f);
		return (
			(Math::abs(u - target.u) <= epsilon) &&
			(Math::abs(v - target.v) <= epsilon));
	}

	/**
	 * 二次元テクスチャ座標が同じでないかどうか
	 * @param target 比較する二次元テクスチャ座標
	 * @return 同じでない値であればtrueを返す
	 */
	inline bool operator !=(const TexCoord2& target) const{
		return ((u != target.u) || (v != target.v));
	}

	/**
	 * 二次元テクスチャ座標が同じでないかどうか
	 * @param target 比較する二次元テクスチャ座標
	 * @param epsilon 誤差
	 * @return 誤差の範囲内で同じでない値であればtrueを返す
	 */
	inline bool notEpsilonEquals(
		const TexCoord2& target, float epsilon) const{
		Assert(epsilon >= 0.f);
		return (
			(Math::abs(u - target.u) > epsilon) ||
			(Math::abs(v - target.v) > epsilon));
	}

	//--------------------------------------------------------------------------
	// その他
	//--------------------------------------------------------------------------
	/**
	 * 文字列化
	 * @return 二次元テクスチャ座標の文字列表記
	 */
	inline String toString() const{
		String returnString;
		returnString.format("( %.8f, %.8f )", u, v);
		return returnString;
	}

	//--------------------------------------------------------------------------
private:

};

//------------------------------------------------------------------------------
} // End of namespace Lamp
#endif // End of TEX_COORD_2_H_
//------------------------------------------------------------------------------
